import 'package:flx_ui/flx_ui.dart';
import 'package:get/get.dart';

typedef TargetTapCallback = void Function();
typedef TargetLongTapCallback = void Function();
typedef TargetDoubleTapCallback = void Function();

class Target extends StatelessWidget {
  final Widget child;

  final TargetTapCallback onTap;
  final TargetLongTapCallback? onLongTap;
  final TargetDoubleTapCallback? onDoubleTap;

  const Target({
    Key? key,
    required this.child,
    required this.onTap,
    this.onLongTap,
    this.onDoubleTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GetBuilder<TargetController>(
      global: false,
      init: TargetController(),
      builder: (controller) => AnimatedBuilder(
        animation: controller.animationController,
        builder: (context, child) => CustomPaint(
          foregroundPainter: TargetWavePainter(
            splashRadius: controller.radiusAnimation.value,
            splashOpacity: controller.opacityAnimation.value,
            color: Colors.white,
            position: controller.tapPosition ?? Offset(0, 0),
          ),
          child: child,
        ),
        child: GestureDetector(
          onTap: onTap,
          onTapUp: (event) async {
            controller.tapPosition = event.localPosition;

            controller.animationController.duration =
                Duration(milliseconds: 200);
            await controller.animationController.forward(from: 0);

            controller.animationController.duration =
                Duration(milliseconds: 350);
            await controller.animationController.reverse();
          },
          onLongPress: onLongTap,
          onDoubleTap: onDoubleTap,
          child: child,
        ),
      ),
    );
  }
}
